package ifneeded tclJBlend 2.1.0 [list apply {dir {
    # If we're running from a bundled enviroment, the shared
    # library has to be copied out to the native filesystem,
    # since the JVM needs to load it, too. It is assumed that
    # either the bundle or the tclJBlend shared library contains
    # the tcl/lang class files from tclJBlend.jar and the JVM can
    # fetch it from there.
    if {[info command "zipfs"] eq "zipfs"} {
        set noe [info nameofexecutable]
        if {[zipfs mount $noe] eq $noe} {
            set clsp $noe
            set ldso [file join $dir libtclJBlend2.1.0.dylib]
            if {[catch {::tcl::unsupported::tempnameforload $ldso} temp]} {
                package require platform::shell
                set temp [platform::shell::TEMP][info sharelibextension]
                # try cleanup later
                set dtmp $temp
            }
            file copy -force -- $ldso $temp
            if {$::tcl_platform(platform) ne "windows"} {
                file attributes $temp -permissions rwxr-xr-x
            }
            lappend ::tclblend_init \
                    "-Dload.tcljblend=[file nativename $temp]"
            set ldso $temp
            append clsp $::tcl_platform(pathSeparator)$ldso
        }
    }
    if {![info exists ldso]} {
        set clsp [file join $dir tclJBlend.jar]
        set ldso [file join $dir libtclJBlend2.1.0.dylib]
        lappend ::tclblend_init \
                "-Dload.tcljblend=[file nativename $ldso]"
    }
    if {$::tcl_platform(platform) eq "windows" &&
        $::tcl_platform(pointerSize) == 4} {
        lappend ::tclblend_init "-Xms128m" "-Xmx256m"
    }
    if {[info exists ::env(CLASSPATH)]} {
        append ::env(CLASSPATH) $::tcl_platform(pathSeparator)$clsp
    } else {
        set ::env(CLASSPATH) $clsp
    }
    try {
        source [file join $dir javaDetect.tcl]
        if {[info exists dtmp] && $::tcl_platform(platform) eq "windows"} {
            # open with delete-on-close to finally get rid of it
            catch {set ftmp [open $temp {RDONLY DELCL}]}
        }
        load $ldso tclblend
        source [file join $dir javalock.tcl]
        source [file join $dir javaUtils.tcl]
    } on error msg {
        return -code error $msg
    } finally {
        # Now, that both Tcl and JVM did load the shared library,
        # it can be removed. However, this can fail on Windows.
        if {[info exists temp]} {
            catch {file delete -force -- $temp}
        }
        if {[info exists ftmp]} {
            catch {close $ftmp}
        }
    }
    package provide tclJBlend 2.1.0
}} $dir]
